#include "AVSTable2D.h"
#include <stdlib.h>
#include <iostream>
#include <string>

using namespace std;

/*
 * This program loads the 19 slices of one preintegration table from
 * disk and creates logarithmic plots of the radiance distribution. 
 * The plots are stored in ppm files.
 */

int main(int argc, char *argv[]) {

	// parse albedo value from command line
	float albedo = 0.0f;
	if (argc < 2) { 

		cout << "Usage: " << argv[0] << " -a <albedo>" << endl;
        return 1;

    } else { 

        for (int i = 1; i < argc; i++) {

			if (i + 1 != argc) {
                if (string(argv[i]) == "-a")                    
                    albedo = static_cast<float>(atof(argv[i + 1]));

                else {
                    cout << "Not enough or invalid arguments, please try again." << endl;
                    return 1;
				}
            }
        }
    }

	// sanity check
	if(albedo < 0.0f || albedo > 1.0f) {
		cout << "Albedo value must be in the range [0,1]." << endl;
		return 1;
	}

	// availability check
	if(albedo != 0.0f && albedo != 0.1f && albedo != 0.2f && albedo != 0.3f && albedo != 0.4f && albedo != 0.5f &&
	   albedo != 0.6f && albedo != 0.7f && albedo != 0.8f && albedo != 0.9f && albedo != 1.0f) {
	   cout << "No table data available. Albedo value must be a multiple of 0.1." << endl;
	   return 1;
	}
	
	// create string identifier
	string albedoString;
	if(albedo == 0.0f) albedoString = string("a00");
	if(albedo == 0.1f) albedoString = string("a01");
	if(albedo == 0.2f) albedoString = string("a02");
	if(albedo == 0.3f) albedoString = string("a03");
	if(albedo == 0.4f) albedoString = string("a04");
	if(albedo == 0.5f) albedoString = string("a05");
	if(albedo == 0.6f) albedoString = string("a06");
	if(albedo == 0.7f) albedoString = string("a07");
	if(albedo == 0.8f) albedoString = string("a08");
	if(albedo == 0.9f) albedoString = string("a09");
	if(albedo == 1.0f) albedoString = string("a10");

	// input table files
	string tab_bw09("AVSTable2D_" + albedoString + "_g-09.tab");
	string tab_bw08("AVSTable2D_" + albedoString + "_g-08.tab");
	string tab_bw07("AVSTable2D_" + albedoString + "_g-07.tab");
	string tab_bw06("AVSTable2D_" + albedoString + "_g-06.tab");
	string tab_bw05("AVSTable2D_" + albedoString + "_g-05.tab");
	string tab_bw04("AVSTable2D_" + albedoString + "_g-04.tab");
	string tab_bw03("AVSTable2D_" + albedoString + "_g-03.tab");
	string tab_bw02("AVSTable2D_" + albedoString + "_g-02.tab");
	string tab_bw01("AVSTable2D_" + albedoString + "_g-01.tab");
	string tab_iso("AVSTable2D_" + albedoString + "_g00.tab");
	string tab_fw01("AVSTable2D_" + albedoString + "_g01.tab");
	string tab_fw02("AVSTable2D_" + albedoString + "_g02.tab");
	string tab_fw03("AVSTable2D_" + albedoString + "_g03.tab");
	string tab_fw04("AVSTable2D_" + albedoString + "_g04.tab");
	string tab_fw05("AVSTable2D_" + albedoString + "_g05.tab");
	string tab_fw06("AVSTable2D_" + albedoString + "_g06.tab");
	string tab_fw07("AVSTable2D_" + albedoString + "_g07.tab");
	string tab_fw08("AVSTable2D_" + albedoString + "_g08.tab");
	string tab_fw09("AVSTable2D_" + albedoString + "_g09.tab");
	
	// output ppm files
	string ppm_bw09("AVSTable2D_" + albedoString + "_g-09.ppm");
	string ppm_bw08("AVSTable2D_" + albedoString + "_g-08.ppm");
	string ppm_bw07("AVSTable2D_" + albedoString + "_g-07.ppm");
	string ppm_bw06("AVSTable2D_" + albedoString + "_g-06.ppm");
	string ppm_bw05("AVSTable2D_" + albedoString + "_g-05.ppm");
	string ppm_bw04("AVSTable2D_" + albedoString + "_g-04.ppm");
	string ppm_bw03("AVSTable2D_" + albedoString + "_g-03.ppm");
	string ppm_bw02("AVSTable2D_" + albedoString + "_g-02.ppm");
	string ppm_bw01("AVSTable2D_" + albedoString + "_g-01.ppm");
	string ppm_iso("AVSTable2D_" + albedoString + "_g00.ppm");
	string ppm_fw01("AVSTable2D_" + albedoString + "_g01.ppm");
	string ppm_fw02("AVSTable2D_" + albedoString + "_g02.ppm");
	string ppm_fw03("AVSTable2D_" + albedoString + "_g03.ppm");
	string ppm_fw04("AVSTable2D_" + albedoString + "_g04.ppm");
	string ppm_fw05("AVSTable2D_" + albedoString + "_g05.ppm");
	string ppm_fw06("AVSTable2D_" + albedoString + "_g06.ppm");
	string ppm_fw07("AVSTable2D_" + albedoString + "_g07.ppm");
	string ppm_fw08("AVSTable2D_" + albedoString + "_g08.ppm");
	string ppm_fw09("AVSTable2D_" + albedoString + "_g09.ppm");

	// read tables
	AVSTable2D table_bw09(tab_bw09);
	AVSTable2D table_bw08(tab_bw08);
	AVSTable2D table_bw07(tab_bw07);
	AVSTable2D table_bw06(tab_bw06);
	AVSTable2D table_bw05(tab_bw05);
	AVSTable2D table_bw04(tab_bw04);
	AVSTable2D table_bw03(tab_bw03);
	AVSTable2D table_bw02(tab_bw02);
	AVSTable2D table_bw01(tab_bw01);
	AVSTable2D table_iso(tab_iso);
	AVSTable2D table_fw01(tab_fw01);
	AVSTable2D table_fw02(tab_fw02);
	AVSTable2D table_fw03(tab_fw03);
	AVSTable2D table_fw04(tab_fw04);
	AVSTable2D table_fw05(tab_fw05);
	AVSTable2D table_fw06(tab_fw06);
	AVSTable2D table_fw07(tab_fw07);
	AVSTable2D table_fw08(tab_fw08);
	AVSTable2D table_fw09(tab_fw09);

	// plot tables
	table_bw09.plotRadianceToPPM(ppm_bw09);
	table_bw08.plotRadianceToPPM(ppm_bw08);
	table_bw07.plotRadianceToPPM(ppm_bw07);
	table_bw06.plotRadianceToPPM(ppm_bw06);
	table_bw05.plotRadianceToPPM(ppm_bw05);
	table_bw04.plotRadianceToPPM(ppm_bw04);
	table_bw03.plotRadianceToPPM(ppm_bw03);
	table_bw02.plotRadianceToPPM(ppm_bw02);
	table_bw01.plotRadianceToPPM(ppm_bw01);
	table_iso.plotRadianceToPPM(ppm_iso);
	table_fw01.plotRadianceToPPM(ppm_fw01);
	table_fw02.plotRadianceToPPM(ppm_fw02);
	table_fw03.plotRadianceToPPM(ppm_fw03);
	table_fw04.plotRadianceToPPM(ppm_fw04);
	table_fw05.plotRadianceToPPM(ppm_fw05);
	table_fw06.plotRadianceToPPM(ppm_fw06);
	table_fw07.plotRadianceToPPM(ppm_fw07);
	table_fw08.plotRadianceToPPM(ppm_fw08);
	table_fw09.plotRadianceToPPM(ppm_fw09);
	
	return 0;
}